//////////////////////////////////////////////////////////
// Sketch library for HC-SR04 (manufacturer unknown)
// Programmed by KEI SAKAKI. http://kei-sakaki.jp/

#include <hc_sr04.h>

HC_SR04::HC_SR04(int TirggerPin, int EchoPin)
{
  this->TirggerPin = TirggerPin;
  this->EchoPin = EchoPin;
  LastDistance = NAN;

  setTemperature(15.0f);  // Typical temperature of 340 meters per second.
  setDurationRange();

  pinMode(TirggerPin, OUTPUT);
  pinMode(EchoPin, INPUT);

  digitalWrite(TirggerPin, LOW);
  delayMicroseconds(10);
}

HC_SR04::~HC_SR04()
{
}

void HC_SR04::setDurationRange(int max, int min)
{
  if(min < MIN_ECHO_DURATION) {
    min = MIN_ECHO_DURATION;
  }
  if(max > MAX_ECHO_DURATION) {
    max = MAX_ECHO_DURATION;
  }
  
  MinDuration = min;
  MaxDuration = max;
}

float HC_SR04::getDistance(bool inch)
{
  int duration;
  float distance;
  
  digitalWrite(TirggerPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(TirggerPin, LOW);

  duration = pulseIn(EchoPin, HIGH, MaxDuration);
  if(duration >= MinDuration && duration <= MaxDuration) {
    distance = duration / 2.0f * SoundSpeed / 10000.0f;
  } else {
    distance = NAN;
  }
  
  LastDistance = distance;
  
  if(distance != NAN && inch) {
    distance /= 2.54f;
  }
  
  return distance;
}

float HC_SR04::getLastDistance(bool inch)
{
  float distance = LastDistance;
  
  if(distance != NAN && inch) {
    distance /= 2.54f;
  }
  
  return distance;
}

void HC_SR04::setTemperature(float Temperature)
{
  this->Temperature = Temperature;
  setSoundSpeed((float) (20.055f * pow(Temperature + 273.15f, 0.5f)));
}

float HC_SR04::getTemperature()
{
  return Temperature;
}

void HC_SR04::setSoundSpeed(float SoundSpeed)
{
  this->SoundSpeed = SoundSpeed;
}

float HC_SR04::getSoundSpeed()
{
  return SoundSpeed;
}
